@@ -4,7 +4,7 @@
 Offline-First Gemini/Web/Gopher/RSS reader and browser
 """
 
-__version__ = "2.5"
+__version__ = "2.6"
 
 # Initial imports and conditional imports {{{
 import argparse
@@ -15,13 +15,22 @@ import shutil
 import sys
 import time
 import urllib.parse
-import netcache
-import opnk
+
 import ansicat
-import offthemes
-from offutils import run, term_width, is_local, mode_url, unmode_url, looks_like_url
-from offutils import xdg
+import netcache
 import offblocklist
+import offthemes
+import opnk
+from offutils import (
+    is_local,
+    looks_like_url,
+    mode_url,
+    run,
+    term_width,
+    unmode_url,
+    xdg,
+    init_config,
+)
 
 try:
     import setproctitle
@@ -123,6 +132,7 @@ _MIME_HANDLERS = {}
 
 
 # GeminiClient Decorators
+# decorator to be sure that self.current_url exists
 def needs_gi(inner):
     def outer(self, *args, **kwargs):
         if not self.current_url:
@@ -134,7 +144,6 @@ def needs_gi(inner):
     outer.__doc__ = inner.__doc__
     return outer
 
-
 class GeminiClient(cmd.Cmd):
     def __init__(self, completekey="tab", synconly=False):
         cmd.Cmd.__init__(self)
@@ -170,6 +179,7 @@ class GeminiClient(cmd.Cmd):
             "history_size": 200,
             "max_size_download": 10,
             "editor": None,
+            #FIXME: download_images_first is not used anywhere in the code
             "download_images_first": True,
             "images_mode": "readable",
             "redirects": True,
@@ -178,6 +188,8 @@ class GeminiClient(cmd.Cmd):
             "search": "gemini://kennedy.gemi.dev/search?%s",
             "accept_bad_ssl_certificates": False,
             "default_protocol": "gemini",
+            "ftr_site_config": None,
+            "preformat_wrap": False,
         }
         self.redirects = offblocklist.redirects
         for i in offblocklist.blocked:
@@ -344,6 +356,8 @@ class GeminiClient(cmd.Cmd):
         params["accept_bad_ssl_certificates"] = self.options[
             "accept_bad_ssl_certificates"
         ]
+        params["ftr_site_config"] = self.options["ftr_site_config"]
+        params["preformat_wrap"] = self.options["preformat_wrap"]
         if mode:
             params["images_mode"] = mode
         else:
@@ -517,7 +531,6 @@ class GeminiClient(cmd.Cmd):
                     value = int(value)
                     print("changing width to ", value)
                     term_width(new_width=value)
-                    self.opencache.cleanup()
                 else:
                     print("%s is not a valid width (integer required)" % value)
             elif value.isnumeric():
@@ -532,6 +545,9 @@ class GeminiClient(cmd.Cmd):
                 except ValueError:
                     pass
             self.options[option] = value
+            #We clean the cache for some options that affect rendering
+            if option in ["preformat_wrap","width"]:
+                self.opencache.cleanup()
 
     def do_theme(self, line):
         """Change the colors of your rendered text.
@@ -560,6 +576,7 @@ class GeminiClient(cmd.Cmd):
                 for k in offthemes.default:
                     valid.append(k)
                 print(valid)
+                return
             else:
                 if le == 1:
                     if element in self.theme.keys():
@@ -569,6 +586,7 @@ class GeminiClient(cmd.Cmd):
                     print("%s is set to %s" % (element, str(value)))
                 else:
                     # Now we parse the colors
+                    toset = None
                     for w in words[1:]:
                         if w not in offthemes.colors.keys():
                             print("%s is not a valid color" % w)
@@ -580,14 +598,24 @@ class GeminiClient(cmd.Cmd):
                             return
                     self.theme[element] = words[1:]
                     self.opencache.cleanup()
-        # now we upadte the prompt
+        # now we update the prompt
         if self.offline_only:
             self.set_prompt("OFF")
         else:
             self.set_prompt("ON")
 
     def do_handler(self, line):
-        """View or set handler commands for different MIME types."""
+        """View or set handler commands for different MIME types.
+        handler MIMETYPE : see handler for MIMETYPE
+        handler MIMETYPE CMD : set handler for MIMETYPE to CMD
+        in the CMD, %s will be replaced by the filename.
+        if no %s, it will be added at the end.
+        MIMETYPE can be the true mimetype or the file extension.
+
+        Examples: 
+            handler application/pdf zathura %s
+            handler .odt lowriter
+            handler docx lowriter"""
         if not line.strip():
             # Show all current handlers
             h = self.opencache.get_handlers()
@@ -988,12 +1016,8 @@ class GeminiClient(cmd.Cmd):
         output += " - python-feedparser   : " + has(ansicat._DO_FEED)
         output += " - python-bs4          : " + has(ansicat._HAS_SOUP)
         output += " - python-readability  : " + has(ansicat._HAS_READABILITY)
-        output += " - timg 1.3.2+         : " + has(ansicat._NEW_TIMG)
-        if ansicat._NEW_CHAFA:
-            output += " - chafa 1.10+         : " + has(ansicat._HAS_CHAFA)
-        else:
-            output += " - chafa               : " + has(ansicat._HAS_CHAFA)
-            output += " - python-pil          : " + has(ansicat._HAS_PIL)
+        output += " - timg 1.3.2+         : " + has(ansicat._HAS_TIMG)
+        output += " - chafa 1.10+         : " + has(ansicat._HAS_CHAFA)
         output += "\nNice to have:\n"
         output += " - python-setproctitle       : " + has(_HAS_SETPROCTITLE)
         clip_support = shutil.which("xsel") or shutil.which("xclip")
@@ -1001,12 +1025,7 @@ class GeminiClient(cmd.Cmd):
         output += " - Wayland clipboard (wl-clipboard): " + has(shutil.which("wl-copy"))
 
         output += "\nFeatures :\n"
-        if ansicat._NEW_CHAFA:
-            output += " - Render images (chafa or timg)              : " + has(
-                ansicat._RENDER_IMAGE
-            )
-        else:
-            output += " - Render images (python-pil, chafa or timg)  : " + has(
+        output += " - Render images (chafa or timg)              : " + has(
                 ansicat._RENDER_IMAGE
             )
         output += " - Render HTML (bs4, readability)             : " + has(
@@ -1090,15 +1109,30 @@ class GeminiClient(cmd.Cmd):
         self._show_lookup(offset=i, end=i + 10)
         self.page_index += 10
 
-    # Stuff that does something to most recently viewed item
     @needs_gi
-    def do_cat(self, *args):
-        """Run most recently visited item through "cat" command."""
-        run(
-            "cat",
-            input=open(self.opencache.get_temp_filename(self.current_url), "rb"),
-            direct_output=True,
-        )
+    def do_feed(self, *args):
+        """Display RSS or Atom feeds linked to the current page."""
+        subs = self.get_renderer().get_subscribe_links()
+        # No feed found
+        if len(subs) == 1:
+            if "rss" in subs[0][1] or "atom" in subs[0][1]:
+                print("Current page is already a feed")
+            else:
+                print("No feed found on current page")
+        # Multiple feeds found
+        elif len(subs) > 2:
+            stri = "Available feeds :\n"
+            counter = 0
+            for s in subs:
+                counter += 1
+                stri += "[%s] %s [%s]\n" % (counter, s[0], s[1])
+            stri += "Which view do you want to see ? >"
+            ans = input(stri)
+            if ans.isdigit() and 0 < int(ans) <= len(subs):
+                self.do_go(subs[int(ans) - 1][0])
+        # Only one feed found
+        else:
+            self.do_go(subs[1][0])
 
     @needs_gi
     def do_view(self, *args):
@@ -1106,8 +1140,6 @@ class GeminiClient(cmd.Cmd):
 previous position.
 Use "view normal" to see the default article view on html page.
 Use "view full" to see a complete html page instead of the article view.
-Use "view feed" to see the the linked feed of the page (in any).
-Use "view feeds" to see available feeds on this page.
 Use "view XX" where XX is a number to view information about link XX.
 (full, feed, feeds have no effect on non-html content)."""
         if self.current_url and args and args[0] != "":
@@ -1117,24 +1149,8 @@ Use "view XX" where XX is a number to view information about link XX.
             elif args[0] in ["normal", "readable"]:
                 self._go_to_url(self.current_url, mode="readable")
             elif args[0] == "feed":
-                subs = self.get_renderer().get_subscribe_links()
-                if len(subs) > 1:
-                    self.do_go(subs[1][0])
-                elif "rss" in subs[0][1] or "atom" in subs[0][1]:
-                    print("%s is already a feed" % u)
-                else:
-                    print("No other feed found on %s" % u)
-            elif args[0] == "feeds":
-                subs = self.get_renderer().get_subscribe_links()
-                stri = "Available views :\n"
-                counter = 0
-                for s in subs:
-                    counter += 1
-                    stri += "[%s] %s [%s]\n" % (counter, s[0], s[1])
-                stri += "Which view do you want to see ? >"
-                ans = input(stri)
-                if ans.isdigit() and 0 < int(ans) <= len(subs):
-                    self.do_go(subs[int(ans) - 1][0])
+                print("view feed is deprecated. Use the command feed directly")
+                self.do_feed()
             elif args[0].isdigit():
                 link_url = self.get_renderer().get_link(int(args[0]))
                 if link_url:
@@ -1153,7 +1169,7 @@ Use "view XX" where XX is a number to view information about link XX.
 
             else:
                 print(
-                    "Valid argument for view are : normal, full, feed, feeds or a number"
+                    "Valid argument for view are : normal, full, source or a number"
                 )
         else:
             self._go_to_url(self.current_url)
@@ -1196,11 +1212,20 @@ Use "view XX" where XX is a number to view information about link XX.
 
     @needs_gi
     def do_shell(self, line):
-        """'cat' most recently visited item through a shell pipeline.
-        '!' is an useful shortcut."""
+        """Send the content of the current page to the shell and pipe it.
+        You are supposed to write what will come after the pipe. For example,
+        if you want to count the number of lines containing STRING in the 
+        current page:
+        > shell grep STRING|wc -l
+        '!' is an useful shortcut.
+        > !grep STRING|wc -l
+        """
+        # input is used if we wand to send something else than current page
+        # to the shell
         tmp = self.opencache.get_temp_filename(self.current_url)
         if tmp:
-            run(line, input=open(tmp, "rb"), direct_output=True)
+            input = open(tmp, "rb")
+            run(line, input=input, direct_output=True)
 
     @needs_gi
     def do_save(self, line):
@@ -1279,22 +1304,46 @@ Use "view XX" where XX is a number to view information about link XX.
             self._go_to_url(last_url, handle=False)
 
     @needs_gi
-    def do_url(self, *args):
-        """Print URL of most recently visited item."""
-        url, mode = unmode_url(self.current_url)
-        print(url)
+    def do_url(self, args):
+        """Print the url of the current page.
+        Use "url XX" where XX is a number to print the url of link XX.
+        "url" can also be piped to the shell, using the pipe "|"
+        """
+        splitted = args.split("|",maxsplit=1)
+        url = None
+        final_url = None
+        if splitted[0].strip().isdigit():
+            link_id = int(splitted[0])
+            link_url = self.get_renderer().get_link(link_id)
+            if link_url:
+                url = link_url
+        else:
+            url = self.current_url
+        if url:
+            final_url, mode = unmode_url(url)
+            print(final_url)
+        if final_url and len(splitted) > 1:
+            run(splitted[1], input=final_url, direct_output=True)
 
     # Bookmarking stuff
     @needs_gi
     def do_add(self, line):
-        """Add the current URL to the list specied as argument.
-        If no argument given, URL is added to Bookmarks."""
+        """Add the current URL to the list specified as argument.
+        If no argument given, URL is added to Bookmarks.
+        You can pass a link number as the second argument to add the link.
+        "add $LIST XX" will add link number XX to $LIST
+        """
         args = line.split()
         if len(args) < 1:
             list = "bookmarks"
             if not self.list_path(list):
                 self.list_create(list)
             self.list_add_line(list)
+        elif len(args) > 1 and args[1].isdigit():
+            link_id = int(args[1])
+            link_url = self.get_renderer().get_link(link_id)
+            if link_url:
+                self.list_add_line(args[0],url=link_url)
         else:
             self.list_add_line(args[0])
 
@@ -1381,14 +1430,16 @@ Use "view XX" where XX is a number to view information about link XX.
     def do_archive(self, args):
         """Archive current page by removing it from every list and adding it to
         archives, which is a special historical list limited in size. It is similar to `move archives`."""
+        u, m = unmode_url(self.current_url)
         for li in self.list_lists():
             if li not in ["archives", "history"]:
-                u, m = unmode_url(self.current_url)
                 deleted = self.list_rm_url(u, li)
                 if deleted:
                     print("Removed from %s" % li)
         self.list_add_top("archives", limit=self.options["archives_size"])
-        print("Archiving: %s" % self.get_renderer().get_page_title())
+        r = self.get_renderer()
+        title = r.get_page_title()
+        print("Archiving: %s" % title)
         print(
             "\x1b[2;34mCurrent maximum size of archives : %s\x1b[0m"
             % self.options["archives_size"]
@@ -2005,6 +2056,12 @@ def main():
     parser.add_argument(
         "--bookmarks", action="store_true", help="start with your list of bookmarks"
     )
+    parser.add_argument(
+        "--command",
+        metavar="COMMAND",
+        nargs="*",
+        help="Launch this command after startup",
+    )
     parser.add_argument(
         "--config-file",
         metavar="FILE",
@@ -2077,36 +2134,6 @@ def main():
     gc = GeminiClient(synconly=args.sync)
     torun_queue = []
 
-    # Interactive if offpunk started normally
-    # False if started with --sync
-    # Queue is a list of command (potentially empty)
-    def read_config(queue, rcfile=None, interactive=True):
-        if not rcfile:
-            rcfile = os.path.join(xdg("config"), "offpunkrc")
-        if os.path.exists(rcfile):
-            print("Using config %s" % rcfile)
-            with open(rcfile, "r") as fp:
-                for line in fp:
-                    line = line.strip()
-                    if (args.bookmarks or args.url) and any(
-                        (line.startswith(x) for x in ("go", "g", "tour", "t"))
-                    ):
-                        if args.bookmarks:
-                            print(
-                                'Skipping rc command "%s" due to --bookmarks option.'
-                                % line
-                            )
-                        else:
-                            print(
-                                'Skipping rc command "%s" due to provided URLs.' % line
-                            )
-                        continue
-                    # We always consider redirect
-                    # for the rest, we need to be interactive
-                    if line.startswith("redirect") or interactive:
-                        queue.append(line)
-        return queue
-
     # Act on args
     if args.bookmarks:
         torun_queue.append("bookmarks")
@@ -2155,20 +2182,20 @@ def main():
             depth = int(args.depth)
         else:
             depth = 1
-        read_config(torun_queue, rcfile=args.config_file, interactive=False)
+        torun_queue += init_config(rcfile=args.config_file, interactive=False)
         for line in torun_queue:
             gc.onecmd(line)
         gc.call_sync(refresh_time=refresh_time, depth=depth, lists=args.url)
     else:
         # We are in the normal mode. First process config file
-        torun_queue = read_config(
-            torun_queue, rcfile=args.config_file, interactive=True
-        )
+        torun_queue += init_config(rcfile=args.config_file,interactive=True)
         print("Welcome to Offpunk!")
         print("Type `help` to get the list of available command.")
         for line in torun_queue:
             gc.onecmd(line)
-
+        if args.command:
+            for cmd in args.command:
+                gc.onecmd(cmd)
         while True:
             try:
                 gc.cmdloop()
